"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const numbers_1 = require("./numbers");
const ntext = (text, plural, count) => Math.abs(count) === 1 ? text : plural;
const getMetricUnit = (metric, count) => {
    switch (metric) {
        case "releases":
            return ntext("release", "releases", count);
        case "drift":
        case "pulse":
        default:
            return ntext("libyear", "libyears", count);
    }
};
const getMatchingPattern = (dependency, overrides) => Object.keys(overrides).find((pattern) => RegExp(pattern).test(dependency));
const isExcused = (dependency, overrides) => Object.entries(overrides).some(([pattern, { defer }]) => RegExp(pattern).test(dependency) && Date.now() < Date.parse(defer));
const isBreach = (value, limit, dependency, overrides) => limit != null && value > limit && !isExcused(dependency, overrides !== null && overrides !== void 0 ? overrides : {});
const printIndividual = (dependencies, threshold, overrides) => {
    const printHelper = (metric, dependency, value, limit) => {
        var _a, _b;
        const threshold = (_b = (_a = overrides === null || overrides === void 0 ? void 0 : overrides[getMatchingPattern(dependency, overrides)]) === null || _a === void 0 ? void 0 : _a[metric]) !== null && _b !== void 0 ? _b : limit;
        if (isBreach(value, threshold, dependency, overrides)) {
            console.error(`${chalk.magenta(metric)}: ${chalk.cyan(dependency)} is ${chalk.red(`${numbers_1.printFloat(value)} ${getMetricUnit(metric, value)}`)} behind; threshold is ${chalk.yellow(numbers_1.printFloat(threshold))}.`);
        }
    };
    dependencies.forEach(({ dependency, drift, pulse, releases }) => {
        printHelper("drift", dependency, drift, threshold === null || threshold === void 0 ? void 0 : threshold.driftIndividual);
        printHelper("pulse", dependency, pulse, threshold === null || threshold === void 0 ? void 0 : threshold.pulseIndividual);
        printHelper("releases", dependency, releases, threshold === null || threshold === void 0 ? void 0 : threshold.releasesIndividual);
    });
};
const printCollective = (totalDrift, totalPulse, totalReleases, threshold) => {
    const logger = (value, limit) => isBreach(value, limit) ? console.error : console.log;
    const message = (metric, value, limit) => {
        const valueStyler = isBreach(value, limit) ? chalk.red : chalk.greenBright;
        const valueMessage = `${chalk.magenta(metric)}: ${{
            drift: "package is",
            pulse: "dependencies are",
            releases: "dependencies are",
        }[metric]} ${valueStyler(`${numbers_1.printFloat(value)} ${getMetricUnit(metric, value)}`)} behind`;
        const limitMessage = `threshold is ${chalk.yellow(numbers_1.printFloat(limit))}`;
        return isBreach(value, limit)
            ? `${valueMessage}; ${limitMessage}.`
            : `${valueMessage}.`;
    };
    logger(totalDrift, threshold === null || threshold === void 0 ? void 0 : threshold.driftCollective)(message("drift", totalDrift, threshold === null || threshold === void 0 ? void 0 : threshold.driftCollective));
    logger(totalPulse, threshold === null || threshold === void 0 ? void 0 : threshold.pulseCollective)(message("pulse", totalPulse, threshold === null || threshold === void 0 ? void 0 : threshold.pulseCollective));
    logger(totalReleases, threshold === null || threshold === void 0 ? void 0 : threshold.releasesCollective)(message("releases", totalReleases, threshold === null || threshold === void 0 ? void 0 : threshold.releasesCollective));
};
exports.print = (dependencies, threshold, overrides) => {
    console.table(dependencies.map(({ dependency, drift, pulse, releases, status, available }) => ({
        dependency,
        drift: numbers_1.printFloat(drift),
        pulse: numbers_1.printFloat(pulse),
        releases,
        status,
        available,
    })));
    console.log();
    const totalDrift = dependencies.reduce((acc, { drift }) => (isNaN(drift) ? acc : acc + drift), 0);
    const totalPulse = dependencies.reduce((acc, { pulse }) => (isNaN(pulse) ? acc : acc + pulse), 0);
    const totalReleases = dependencies.reduce((acc, { releases }) => (isNaN(releases) ? acc : acc + releases), 0);
    const breaches = {
        driftCollective: isBreach(totalDrift, threshold === null || threshold === void 0 ? void 0 : threshold.driftCollective),
        driftIndividual: dependencies.some(({ dependency, drift }) => {
            var _a, _b;
            return isBreach(drift, (_b = (_a = overrides === null || overrides === void 0 ? void 0 : overrides[getMatchingPattern(dependency, overrides)]) === null || _a === void 0 ? void 0 : _a.drift) !== null && _b !== void 0 ? _b : threshold === null || threshold === void 0 ? void 0 : threshold.driftIndividual, dependency, overrides);
        }),
        pulseCollective: isBreach(totalPulse, threshold === null || threshold === void 0 ? void 0 : threshold.pulseCollective),
        pulseIndividual: dependencies.some(({ dependency, pulse }) => {
            var _a, _b;
            return isBreach(pulse, (_b = (_a = overrides === null || overrides === void 0 ? void 0 : overrides[getMatchingPattern(dependency, overrides)]) === null || _a === void 0 ? void 0 : _a.pulse) !== null && _b !== void 0 ? _b : threshold === null || threshold === void 0 ? void 0 : threshold.pulseIndividual, dependency, overrides);
        }),
        releasesCollective: isBreach(totalReleases, threshold === null || threshold === void 0 ? void 0 : threshold.releasesCollective),
        releasesIndividual: dependencies.some(({ dependency, releases }) => {
            var _a, _b;
            return isBreach(releases, (_b = (_a = overrides === null || overrides === void 0 ? void 0 : overrides[getMatchingPattern(dependency, overrides)]) === null || _a === void 0 ? void 0 : _a.releases) !== null && _b !== void 0 ? _b : threshold === null || threshold === void 0 ? void 0 : threshold.releasesIndividual, dependency, overrides);
        }),
    };
    if (breaches.driftIndividual ||
        breaches.pulseIndividual ||
        breaches.releasesIndividual) {
        console.log(chalk.bold("# Individual"));
        printIndividual(dependencies, threshold, overrides);
        console.log();
    }
    console.log(chalk.bold("# Collective"));
    printCollective(totalDrift, totalPulse, totalReleases, threshold);
    console.log();
    if (Object.values(breaches).includes(true)) {
        process.exit(1);
    }
};
